#!/bin/bash
####################################################
# script to check when a domain name expires.
# it works only on UNIX system
# you need to install whois on system
# yum install whois - for centos OS
#
# 2020-11-24: Bugfixes detailed in commit /L
#
# @TODO Note on flags: Since -w and -c are hardcoded
# to be in position 4 and 6 respectively, you must
# call plugin with syntax "-h ... -w ... -c ..."
#
# xargs is used to strip whitespace from strings
####################################################

PROGRAM=${0##*/}
PROGPATH=${0%/*}
# shellcheck source=utils.sh
. "$PROGPATH"/utils.sh


# default warn/crit days
if [ -z "$4" ];
then
	warning=30
else warning=$4
fi

if [ -z "$6" ];
then
	critical=7
else critical=$6
fi

# arguments from utils.sh
domain=$2

# Checking if you have whois
WHOIS=/usr/bin/whois
if [ ! -e $WHOIS ];
then
	echo "/usr/bin/whois is missing: please 'yum install whois'"
	exit
fi

# Checking if script can be executed
if [ -z $? ] || [ -z "$1" ] || [ -z "$domain" ];
then
	echo "Usage: $PROGRAM -d <domain> [-w <days>] [-c <days>]"
	exit
fi

# checking .name and defining whois server
DLTYPE=$(echo "$domain" | cut -d '.' -f2)

if [ "$DLTYPE" == 'edu' ];
then
	TYPE=educause
	WHOIS_SERVER="whois.educause.edu"
elif [ "$DLTYPE" == 'com' ] || [ "$DLTYPE" == 'net' ];
then
	TYPE=internic
	WHOIS_SERVER="whois.internic.org"
elif [ "$DLTYPE" == 'org' ];
then
	TYPE=pir
	WHOIS_SERVER="whois.pir.org"
elif [ "$DLTYPE" == 'in' ];
then
	TYPE=in
	WHOIS_SERVER="whois.registry.in"
elif [ "$DLTYPE" == 'co' ];
then
	TYPE=nic
	WHOIS_SERVER="whois.nic.uk"
elif [ "$DLTYPE" == 'biz' ];
then
	TYPE=neulevel
	WHOIS_SERVER="whois.neulevel.biz"
elif [ "$DLTYPE" == 'info' ];
then
	TYPE=afilias
	WHOIS_SERVER="whois.afilias.info"
elif [ "$DLTYPE" == 'ru' ];
then
	TYPE=russia
	WHOIS_SERVER="whois.nic.ru"
elif [ "$DLTYPE" == 'dk' ];
then
	TYPE=dk-hostmaster
	WHOIS_SERVER="whois.dk-hostmaster.dk"
elif [ "$DLTYPE" == 'se' ];
then
    TYPE=iis
    WHOIS_SERVER="whois.iis.se"
elif [ "$DLTYPE" == 'fi' ];
then
    TYPE=ficora
    WHOIS_SERVER="whois.ficora.fi"
elif [ "$DLTYPE" == 'ie' ];
then
    TYPE=domainregistry-ie
    WHOIS_SERVER="whois.domainregistry.ie"
elif [ "$DLTYPE" == 'us' ];
then
    TYPE=nic-us
    WHOIS_SERVER="whois.nic.us"
elif [ "$DLTYPE" == 'cn' ];
then
    TYPE=cnnic
    WHOIS_SERVER="whois.cnnic.cn"
elif [ "$DLTYPE" == 'nu' ];
then
    TYPE=nunames
    WHOIS_SERVER="whois.nic.nu"
elif [ "$DLTYPE" == 'cc' ];
then
    TYPE=cc
    WHOIS_SERVER="whois.nic.cc"
elif [ "$DLTYPE" == 'io' ];
then
    TYPE=cc
    WHOIS_SERVER="whois.nic.io"
else
	echo "Error: Plugin cannot resolve TLD '$DLTYPE'"
	exit "$STATE_UNKNOWN"
fi

# do whois

FILE=/tmp/domain_$2.txt
$WHOIS -h $WHOIS_SERVER "$domain" > "$FILE"

# expiration formats

# Common standard
common=('internic' 'pir' 'in' 'neulevel' 'afilias' 'domainregistry-ie' \
'nic-us' 'cc' 'io')

for i in ${common[*]}; do
	if [[ "$i" = "$TYPE" ]]; then
	# datestring will be something like '2028-09-14T04:00:00Z'
	datestring=$(cat "$FILE" | awk '/Registry Expiry Date:/' | cut -d ':' -f2)
	# we want something like '2028-09-14'
	substr=${datestring:1:10}
	day=$(echo $substr | cut -d '-' -f3)
	month=$(echo $substr | cut -d '-' -f2)
	year=$(echo $substr | cut -d '-' -f1)
	expiration=$year-$month-$day
	fi
done
# for co.uk domains

if [ $TYPE == 'educause' ];
then
expiration=$(cat "$FILE" | awk '/Domain expires:/' | cut -d ':' -f2 | xargs)
fi

if [ $TYPE == 'nic' ];
then
expiration=$(cat "$FILE" | awk '/Expiry date:/' | cut -d ':' -f2)
fi

#for .ru domains
if [ $TYPE == 'russia' ];
then
day=$(cat "$FILE" | awk '/paid-till:/' | cut -d ':' -f2 | cut -d '.' -f3)
month=$(cat "$FILE" | awk '/paid-till:/' | cut -d ':' -f2 | cut -d '.' -f2)
year=$(cat "$FILE" | awk '/paid-till:/' | cut -d ':' -f2 | cut -d ' ' -f5 | cut -d '.' -f1)
expiration=$year-$month-$day
fi

#for .dk domains
if [ $TYPE == 'dk-hostmaster' ];
then
day=$(cat "$FILE" | awk '/Expires:/' | cut -d '-' -f3)
month=$(cat "$FILE" | awk '/Expires:/' | cut -d '-' -f2)
year=$(cat "$FILE" | awk '/Expires:/' | cut -d ' ' -f15 | cut -d '-' -f1)
expiration=$year-$month-$day
fi

#for .se domains
if [ $TYPE == 'iis' ];
then
dos2unix -q "$FILE"
day=$(cat "$FILE" | awk '/expires:/' | cut -d ':' -f2 | cut -d '-' -f3)
month=$(cat "$FILE" | awk '/expires:/' | cut -d ':' -f2 | cut -d '-' -f2)
year=$(cat "$FILE" | awk '/expires:/' | sed 's/^[a-zA-Z\ :]*//g' | cut -d '-' -f1)
expiration=$year-$month-$day
fi

#for .fi domains
if [ $TYPE == 'ficora' ];
then
dos2unix -q "$FILE"
day=$(cat "$FILE" | awk '/expires/' | cut -d ':' -f2 | cut -d ' ' -f2 | cut -d '.' -f1)
month=$(cat "$FILE" | awk '/expires/' | cut -d ':' -f2 | cut -d ' ' -f2 | cut -d '.' -f2)
year=$(cat "$FILE" | awk '/expires/' | cut -d ':' -f2 |  cut -d ' ' -f2 | cut -d '.' -f3)
expiration=$year-$month-$day
fi

#for .cn domains
if [[ $TYPE == 'cnnic' ]];
then
day=$(cat "$FILE" | awk '/Expiration Time:/' | cut -d ' ' -f3 | cut -d '-' -f3)
month=$(cat "$FILE" | awk '/Expiration Time:/' | cut -d ' ' -f3 | cut -d '-' -f2)
year=$(cat "$FILE" | awk '/Expiration Time:/' | cut -d ' ' -f3 | cut -d '-' -f1)
expiration=$year-$month-$day
fi

#for .nu domains
if [ $TYPE == 'nunames' ];
then
day=$(cat "$FILE" | awk '/expires/' | cut -d ':' -f2 | cut -d '-' -f3 | xargs )
month=$(cat "$FILE" | awk '/expires/' | cut -d ':' -f2 | cut -d '-' -f2 | xargs )
year=$(cat "$FILE" | awk '/expires/' | cut -d ':' -f2 | cut -d '-' -f1 | xargs )
expiration=$year-$month-$day
fi

rm "$FILE"

if [ -z "$expiration" ];
then
	echo "UNKNOWN - Could not retrieve expiration time from whois."
	exit "$STATE_UNKNOWN"
fi

# expiration counting
expiration_date=$(date +%s --date="$expiration")
NOW=$(date +%s)
DAYSLEFT=$(((expiration_date-NOW)/86400))

# alerts
if [[ $DAYSLEFT -lt 0 ]];
then
	echo "CRITICAL - Domain is expired"
	exit "$STATE_CRITICAL"
elif [[ $DAYSLEFT -lt "$critical" ]];
then
	echo "CRITICAL - Domain will expire in $DAYSLEFT days"
	exit "$STATE_CRITICAL"
elif [[ $DAYSLEFT -lt "$warning" ]];
then
	echo "WARNING - Domain will expire after $DAYSLEFT days on $expiration"
	exit "$STATE_WARNING"
fi

echo "OK - Domain will expire on $expiration"
exit "$STATE_OK"
